// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;
using Smdn.Net.Imap4.Client.Transaction;

namespace Smdn.Net.Imap4.Client.Session {
  internal class ImapCapabilityResponseProcessor : ImapResponseProcessorBase {
    public ImapCapabilityResponseProcessor(ImapSession session)
      : base(session)
    {
    }

    // 6.2.2. AUTHENTICATE Command
    //       A server MAY include a CAPABILITY response code in the tagged OK
    //       response of a successful AUTHENTICATE command in order to send
    //       capabilities automatically.

    // 6.2.3. LOGIN Command
    //       A server MAY include a CAPABILITY response code in the tagged OK
    //       response to a successful LOGIN command in order to send
    //       capabilities automatically.  It is unnecessary for a client to
    //       send a separate CAPABILITY command if it recognizes these
    //       automatic capabilities.

    // 7.1. Server Responses - Status Responses
    //       CAPABILITY
    //          Followed by a list of capabilities.  This can appear in the
    //          initial OK or PREAUTH response to transmit an initial
    //          capabilities list.  This makes it unnecessary for a client to
    //          send a separate CAPABILITY command if it recognizes this
    //          response.
    public override bool Process(ImapResponse response, IImapTransaction t)
    {
      var status = response as ImapStatusResponse;

      if (status == null)
        return false;

      if (status.ResponseText.Code != ImapResponseCode.Capability)
        return false;

      if (status is ImapTaggedStatusResponse && (status as ImapTaggedStatusResponse).Condition != ImapResponseCondition.Ok)
        return false;

      Session.SetServerCapabilities(ImapResponseTextConverter.FromCapability(status.ResponseText));

      return true;
    }
  }
}