// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client.Transaction.BuiltIn {
  internal class GenericCommandTransaction : ImapTransactionBase<ImapTransactionResult<ImapDataResponse[]>> {
    public GenericCommandTransaction(ImapConnection connection, string command)
      : base(connection)
    {
      this.command = command;
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessGenericCommand;
    }

    private void ProcessGenericCommand()
    {
      if (Request.Arguments.ContainsKey("arguments"))
        SendCommand(command, ProcessReceiveResponse, Request.Arguments["arguments"]);
      else
        SendCommand(command, ProcessReceiveResponse);
    }

    protected override void OnDataResponseReceived(ImapDataResponse data)
    {
      dataResponses.Add(data);

      base.OnDataResponseReceived(data);
    }

    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
    {
      if (tagged.Condition == ImapResponseCondition.Ok) {
        Finish(new ImapTransactionResult<ImapDataResponse[]>(dataResponses.ToArray(), tagged.ResponseText.Text));
        dataResponses.Clear();
      }
      else {
        base.OnTaggedStatusResponseReceived(tagged);
      }
    }

    private /*readonly*/ string command;
    private List<ImapDataResponse> dataResponses = new List<ImapDataResponse>();
  }
}
