// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Net.Imap4 {
  // 7.4.2. FETCH Response
  //       BODYSTRUCTURE
  //         The basic fields of a non-multipart body part are in the
  //         following order:
  //
  //         body type
  //            A string giving the content media type name as defined in
  //            [MIME-IMB].
  //
  //         body subtype
  //            A string giving the content subtype name as defined in
  //            [MIME-IMB].
  //
  //         body parameter parenthesized list
  //            A parenthesized list of attribute/value pairs [e.g., ("foo"
  //            "bar" "baz" "rag") where "bar" is the value of "foo" and
  //            "rag" is the value of "baz"] as defined in [MIME-IMB].
  //
  //         body id
  //            A string giving the content id as defined in [MIME-IMB].
  //
  //         body description
  //            A string giving the content description as defined in
  //            [MIME-IMB].
  //
  //         body encoding
  //            A string giving the content transfer encoding as defined in
  //            [MIME-IMB].
  //
  //         body size
  //            A number giving the size of the body in octets.  Note that
  //            this size is the size in its transfer encoding and not the
  //            resulting size after any decoding.
  //
  //         A body type of type MESSAGE and subtype RFC822 contains,
  //         immediately after the basic fields, the envelope structure,
  //         body structure, and size in text lines of the encapsulated
  //         message.
  //
  //         A body type of type TEXT contains, immediately after the basic
  //         fields, the size of the body in text lines.  Note that this
  //         size is the size in its content transfer encoding and not the
  //         resulting size after any decoding.
  public abstract class ImapBodyStructureBase {
    public string Subtype {
      get { return subtype; }
    }

    public abstract bool IsMultiPart { get; }

    protected ImapBodyStructureBase(string subtype)
    {
      if (subtype == null)
        throw new ArgumentNullException("subtype");

      this.subtype = subtype;
    }

    private /*readonly*/ string subtype;
  }
}
/*
body            = "(" (body-type-1part / body-type-mpart) ")"
body-extension  = nstring / number /
                   "(" body-extension *(SP body-extension) ")"
                    ; Future expansion.  Client implementations
                    ; MUST accept body-extension fields.  Server
                    ; implementations MUST NOT generate
                    ; body-extension fields except as defined by
                    ; future standard or standards-track
                    ; revisions of this specification.
body-ext-1part  = body-fld-md5 [SP body-fld-dsp [SP body-fld-lang
                  [SP body-fld-loc *(SP body-extension)]]]
                    ; MUST NOT be returned on non-extensible
                    ; "BODY" fetch
body-ext-mpart  = body-fld-param [SP body-fld-dsp [SP body-fld-lang
                  [SP body-fld-loc *(SP body-extension)]]]
                    ; MUST NOT be returned on non-extensible
                    ; "BODY" fetch
body-fields     = body-fld-param SP body-fld-id SP body-fld-desc SP
                  body-fld-enc SP body-fld-octets
body-fld-desc   = nstring
body-fld-dsp    = "(" string SP body-fld-param ")" / nil
body-fld-enc    = (DQUOTE ("7BIT" / "8BIT" / "BINARY" / "BASE64"/
                  "QUOTED-PRINTABLE") DQUOTE) / string
body-fld-id     = nstring
body-fld-lang   = nstring / "(" string *(SP string) ")"
body-fld-loc    = nstring
body-fld-lines  = number
body-fld-md5    = nstring
body-fld-octets = number
body-fld-param  = "(" string SP string *(SP string SP string) ")" / nil
body-type-1part = (body-type-basic / body-type-msg / body-type-text)
                  [SP body-ext-1part]
body-type-basic = media-basic SP body-fields
                    ; MESSAGE subtype MUST NOT be "RFC822"
body-type-mpart = 1*body SP media-subtype
                  [SP body-ext-mpart]
body-type-msg   = media-message SP body-fields SP envelope
                  SP body SP body-fld-lines
body-type-text  = media-text SP body-fields SP body-fld-lines

media-basic     = ((DQUOTE ("APPLICATION" / "AUDIO" / "IMAGE" /
                  "MESSAGE" / "VIDEO") DQUOTE) / string) SP
                  media-subtype
                    ; Defined in [MIME-IMT]
media-message   = DQUOTE "MESSAGE" DQUOTE SP DQUOTE "RFC822" DQUOTE
                    ; Defined in [MIME-IMT]
media-subtype   = string
                    ; Defined in [MIME-IMT]
media-text      = DQUOTE "TEXT" DQUOTE SP media-subtype
                    ; Defined in [MIME-IMT]
 */