// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Net.Imap4 {
  public class ImapSinglePartBodyStructure : ImapBodyStructureBase {
    public override bool IsMultiPart {
      get { return false; }
    }

    /*
     * basic fields
     */
    public string Type {
      get { return type; }
    }

    public Dictionary<string, string> Parameters {
      get { return parameters; }
    }

    public string Id {
      get { return id; }
    }

    public string Description {
      get { return description; }
    }

    public string Encoding {
      get { return encoding; }
    }

    public long Size {
      get { return size; }
    }

    /*
     * type MESSAGE
     */
    public ImapEnvelope Envelope { // of encapsulated message
      get { return envelope; }
    }

    public ImapBodyStructureBase BodyStructure { // of encapsulated message
      get { return bodyStructure; }
    }

    /*
     * type TEXT, type MESSAGE
     */
    public long Lines { // the size of the body in text lines(type TEXT)
                        // size in text lines of the encapsulated message(type MESSAGE)
      get { return lines; }
    }

    public ImapSinglePartBodyStructure(string type,
                                       string subtype,
                                       Dictionary<string, string> parameters,
                                       string id,
                                       string description,
                                       string encoding,
                                       long size,
                                       ImapEnvelope envelope,
                                       ImapBodyStructureBase bodyStructure,
                                       long lines)
      : base(subtype)
    {
      if (type == null)
        throw new ArgumentNullException("type");
      if (size < 0)
        throw new ArgumentOutOfRangeException("size", "must be zero or positive number");
      if (size < 0)
        throw new ArgumentOutOfRangeException("lines", "must be zero or positive number");

      this.type           = type;
      if (parameters == null)
        this.parameters   = new Dictionary<string, string>();
      else
        this.parameters   = new Dictionary<string, string>(parameters);
      this.id             = id;
      this.description    = description;
      this.encoding       = encoding;
      this.size           = size;
      this.envelope       = envelope;
      this.bodyStructure  = bodyStructure;
      this.lines          = lines;
    }

    public override string ToString()
    {
      return string.Format("{{Type={0}, Subtype={1}, Parameters={2}, Id={3}, Description={4}, Encoding={5}, Size={6}, Envelope={7}, BodyStructure={8}, Lines={9}}}",
                           type,
                           Subtype,
                           parameters.Count,
                           id,
                           description,
                           encoding,
                           size,
                           envelope,
                           bodyStructure,
                           lines);
    }

    private /*readonly*/ string type;
    private /*readonly*/ Dictionary<string, string> parameters;
    private /*readonly*/ string id;
    private /*readonly*/ string description;
    private /*readonly*/ string encoding;
    private readonly long size;
    private /*readonly*/ ImapEnvelope envelope;
    private /*readonly*/ ImapBodyStructureBase bodyStructure;
    private readonly long lines;
  }
}