using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapMatchedSequenceSetTests {
    [Test]
    public void TestIsSavedResult()
    {
      Assert.IsFalse((new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).IsSavedResult);
      Assert.IsTrue(ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).IsSavedResult);
    }

    [Test]
    public void TestIsEmpty()
    {
      Assert.IsTrue(ImapMatchedSequenceSet.CreateEmpty(true).IsEmpty);
      Assert.IsFalse(ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).IsEmpty);
    }

    [Test]
    public void TestIsSingle()
    {
      var singleSet = ImapSequenceSet.CreateSet(1L);

      Assert.IsTrue(singleSet.IsSingle);
      Assert.IsFalse(ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(singleSet)).IsSingle);
      Assert.IsTrue((new ImapMatchedSequenceSet(singleSet)).IsSingle);
    }

    [Test]
    public void TestToArray()
    {
      var originalSet = ImapSequenceSet.CreateRangeSet(1, 3);

      Assert.AreEqual(originalSet.ToArray(), (new ImapMatchedSequenceSet(originalSet)).ToArray());
    }

    [Test, ExpectedException(typeof(NotSupportedException))]
    public void TestToArraySavedResult()
    {
      ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).ToArray();
    }

    [Test]
    public void TestToNumber()
    {
      Assert.AreEqual(1L, (new ImapMatchedSequenceSet(ImapSequenceSet.CreateSet(1L))).ToNumber());
    }

    [Test, ExpectedException(typeof(NotSupportedException))]
    public void TestToNumberSavedResult()
    {
      ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(ImapSequenceSet.CreateSet(1L))).ToNumber();
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("*", (new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).ToString());
    }

    [Test]
    public void TestToStringSavedResult()
    {
      Assert.AreEqual("$", ImapMatchedSequenceSet.CreateSavedResult(new ImapMatchedSequenceSet(ImapSequenceSet.CreateAllSet())).ToString());
    }
  }
}
