using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapMessageFlagTests {
    [Test, ExpectedException(typeof(ArgumentNullException))]
    public void TestValidateKeywordNull()
    {
      ImapMessageFlag.ValidateKeyword(null);
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestValidateKeywordEmpty()
    {
      ImapMessageFlag.ValidateKeyword(string.Empty);
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestValidateKeywordContainsCtrl()
    {
      ImapMessageFlag.ValidateKeyword("keyword\n");
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestValidateKeywordSystemFlags()
    {
      ImapMessageFlag.ValidateKeyword("\\Deleted");
    }

    [Test]
    public void TestValidateKeywordValid()
    {
      Assert.AreEqual("a", ImapMessageFlag.ValidateKeyword("a"));
      Assert.AreEqual("$label", ImapMessageFlag.ValidateKeyword("$label"));
      Assert.AreEqual("<label>", ImapMessageFlag.ValidateKeyword("<label>"));
    }
  }
}
