using System;
using System.Net;
using NUnit.Framework;

namespace Smdn.Security.Authentication.Sasl.Client {
  [TestFixture]
  public class SaslClientMechanismTests {
    [Test]
    public void TestGetAvailableMechanisms()
    {
      var mechanisms = SaslClientMechanism.GetAvailableMechanisms();

      Assert.IsNotNull(mechanisms);
      Assert.Greater(mechanisms.Length, 0);
      CollectionAssert.Contains(mechanisms, "LOGIN");
      CollectionAssert.Contains(mechanisms, "PLAIN");
      CollectionAssert.Contains(mechanisms, "ANONYMOUS");
      CollectionAssert.Contains(mechanisms, "DIGEST-MD5");
      CollectionAssert.Contains(mechanisms, "CRAM-MD5");
    }

    [Test]
    public void TestCreate()
    {
      var mechanism = SaslClientMechanism.Create("PLAIN");

      Assert.IsNotNull(mechanism);
      Assert.IsInstanceOfType(typeof(PlainMechanism), mechanism);
    }

    [Test, ExpectedException(typeof(SaslMechanismNotSupportedException))]
    public void TestCreateNotSupported()
    {
      SaslClientMechanism.Create("X-UNKNOWN");
    }

    [Test]
    public void TestCreateNtlm()
    {
      try {
        var mechanism = SaslClientMechanism.Create("NTLM");

        Assert.IsNotNull(mechanism);
        Assert.IsInstanceOfType(typeof(NTLMMechanism), mechanism);
      }
      catch (SaslMechanismNotSupportedException) {
        Assert.Ignore("NTLM not supported");
      }
      catch (SaslException) {
        Assert.Ignore("NTLM unavailable");
      }
    }

    [Test, ExpectedException(typeof(ObjectDisposedException))]
    public void TestGetCredentialAfterDisposed()
    {
      var mechanism = SaslClientMechanism.Create("PLAIN");

      mechanism.Credential = new NetworkCredential("test", "test");

      mechanism.Dispose();

      Assert.IsNull(mechanism.Credential);
    }

    [Test, ExpectedException(typeof(ObjectDisposedException))]
    public void TestExchangeAfterDisposed()
    {
      var mechanism = SaslClientMechanism.Create("PLAIN");

      mechanism.Credential = new NetworkCredential("test", "test");

      mechanism.Dispose();

      byte[] clientResponse;

      mechanism.Exchange(null, out clientResponse);
    }
  }
}
