using System;
using System.Net;

namespace Smdn.Applications.TundereBird {
  public abstract class AccountBase {
    public string Name {
      get; private set;
    }

    public Uri ServerUri {
      get; private set;
    }

    public int Timeout {
      get; private set;
    }

    protected abstract ICredentials Credentials { get; }

    protected AccountBase(string name, Uri serverUri, int timeout)
    {
      this.Name = name;
      this.ServerUri = serverUri;
      this.Timeout = timeout;
    }

    protected WebRequest CreateWebRequest(Uri requestUri, string method)
    {
      if (requestUri == null)
        requestUri = ServerUri;
      else
        requestUri = new Uri(ServerUri, ServerUri.MakeRelativeUri(requestUri));

      var request = WebRequest.Create(requestUri);

      request.Credentials = Credentials;
      request.Timeout = Timeout;

      if (method != null)
        request.Method = method;

      return request;
    }

    public void Request(string method)
    {
      Request(null, method);
    }

    public void Request(Uri requestUri, string method)
    {
      using (var response = CreateWebRequest(requestUri, method).GetResponse()) {
        OnGotResponse(response);
      }
    }

    public TResult Request<TResult>(Uri requestUri, Func<WebResponse, TResult> action)
    {
      using (var response = CreateWebRequest(requestUri, null).GetResponse()) {
        OnGotResponse(response);

        if (action != null)
          return action(response);
      }

      return default(TResult);
    }

    protected virtual void OnGotResponse(WebResponse response)
    {
    }
  }
}
