using System;
using System.Net;

using Smdn.Net.Pop3;
using Smdn.Net.Pop3.Client;
using Smdn.Net.Pop3.WebClients;

namespace Smdn.Applications.TundereBird {
  public class PopAccount : AccountBase {
    protected override ICredentials Credentials {
      get { return credential; }
    }

    public bool UseTlsIfAvailable {
      get; private set;
    }

    public PopCapabilityList ServerCapabilities {
      get; private set;
    }

    public PopAccount(string name, Uri serverUri, ICredentials credential, int timeout, bool useTlsIfAvailable)
      : base(name, serverUri, timeout)
    {
      this.credential = credential;
      this.UseTlsIfAvailable = useTlsIfAvailable;
    }

    public PopWebRequest CreateRequest(Uri requestUri, string method)
    {
      var request = CreateWebRequest(requestUri, method) as PopWebRequest;

      request.UseTlsIfAvailable = UseTlsIfAvailable;

      return request;
    }

    public TResult Request<TResult>(Func<PopWebResponse, TResult> action)
    {
      return Request(null, null, action);
    }

    public TResult Request<TResult>(Uri requestUri, Func<PopWebResponse, TResult> action)
    {
      return Request(requestUri, null, action);
    }

    public TResult Request<TResult>(Uri requestUri, string method, Func<PopWebResponse, TResult> action)
    {
      using (var response = CreateRequest(requestUri, method).GetResponse() as PopWebResponse) {
        OnGotResponse(response);

        if (action != null)
          return action(response);
      }

      return default(TResult);
    }

    protected override void OnGotResponse(WebResponse response)
    {
      var r = response as PopWebResponse;

      if (!serverInfoRetrieved) {
        ServerCapabilities = r.ServerCapabilities;

        serverInfoRetrieved = true;
      }

      base.OnGotResponse(response);
    }

    private bool serverInfoRetrieved = false;
    private ICredentials credential;
  }
}
